@@ -1,4 +1,5 @@
 from io import BytesIO
+from tempfile import SpooledTemporaryFile
 
 from asgiref.sync import AsyncToSync, sync_to_async
 
@@ -11,13 +12,13 @@ class WsgiToAsgi:
     def __init__(self, wsgi_application):
         self.wsgi_application = wsgi_application
 
-    def __call__(self, scope):
+    async def __call__(self, scope, receive, send):
         """
         ASGI application instantiation point.
         We return a new WsgiToAsgiInstance here with the WSGI app
         and the scope, ready to respond when it is __call__ed.
         """
-        return WsgiToAsgiInstance(self.wsgi_application, scope)
+        await WsgiToAsgiInstance(self.wsgi_application)(scope, receive, send)
 
 
 class WsgiToAsgiInstance:
@@ -25,34 +26,42 @@ class WsgiToAsgiInstance:
     Per-socket instance of a wrapped WSGI application
     """
 
-    def __init__(self, wsgi_application, scope):
+    def __init__(self, wsgi_application):
         self.wsgi_application = wsgi_application
-        self.scope = scope
         self.response_started = False
 
-    async def __call__(self, receive, send):
-        # Alright, wait for the http.request message
-        message = await receive()
-        if message["type"] != "http.request":
-            raise ValueError("WSGI wrapper received a non-HTTP-request message")
-        # Wrap send so it can be called from the subthread
-        self.sync_send = AsyncToSync(send)
-        # Call the WSGI app
-        await self.run_wsgi_app(message)
+    async def __call__(self, scope, receive, send):
+        if scope["type"] != "http":
+            raise ValueError("WSGI wrapper received a non-HTTP scope")
+        self.scope = scope
+        with SpooledTemporaryFile(max_size=65536) as body:
+            # Alright, wait for the http.request messages
+            while True:
+                message = await receive()
+                if message["type"] != "http.request":
+                    raise ValueError("WSGI wrapper received a non-HTTP-request message")
+                body.write(message.get("body", b""))
+                if not message.get("more_body"):
+                    break
+            body.seek(0)
+            # Wrap send so it can be called from the subthread
+            self.sync_send = AsyncToSync(send)
+            # Call the WSGI app
+            await self.run_wsgi_app(body)
 
-    def build_environ(self, scope, message):
+    def build_environ(self, scope, body):
         """
-        Builds a scope and request message into a WSGI environ object.
+        Builds a scope and request body into a WSGI environ object.
         """
         environ = {
             "REQUEST_METHOD": scope["method"],
-            "SCRIPT_NAME": "",
+            "SCRIPT_NAME": scope.get("root_path", ""),
             "PATH_INFO": scope["path"],
             "QUERY_STRING": scope["query_string"].decode("ascii"),
             "SERVER_PROTOCOL": "HTTP/%s" % scope["http_version"],
             "wsgi.version": (1, 0),
             "wsgi.url_scheme": scope.get("scheme", "http"),
-            "wsgi.input": BytesIO(message.get("body", b"")),
+            "wsgi.input": body,
             "wsgi.errors": BytesIO(),
             "wsgi.multithread": True,
             "wsgi.multiprocess": True,
@@ -61,10 +70,14 @@ class WsgiToAsgiInstance:
         # Get server name and port - required in WSGI, not in ASGI
         if "server" in scope:
             environ["SERVER_NAME"] = scope["server"][0]
-            environ["SERVER_PORT"] = scope["server"][1]
+            environ["SERVER_PORT"] = str(scope["server"][1])
         else:
             environ["SERVER_NAME"] = "localhost"
-            environ["SERVER_PORT"] = 80
+            environ["SERVER_PORT"] = "80"
+
+        if "client" in scope:
+            environ["REMOTE_ADDR"] = scope["client"][0]
+
         # Go through headers and make them into environ entries
         for name, value in self.scope.get("headers", []):
             name = name.decode("latin1")
@@ -90,13 +103,15 @@ class WsgiToAsgiInstance:
             raise exc_info[1].with_traceback(exc_info[2])
         # Don't allow re-calling without exc_info
         if hasattr(self, "response_start") and exc_info is None:
-            raise ValueError("You cannot call start_response a second time without exc_info")
+            raise ValueError(
+                "You cannot call start_response a second time without exc_info"
+            )
         # Extract status code
         status_code, _ = status.split(" ", 1)
         status_code = int(status_code)
         # Extract headers
         headers = [
-            (name.encode("ascii"), value.encode("ascii"))
+            (name.lower().encode("ascii"), value.encode("ascii"))
             for name, value in response_headers
         ]
         # Build and send response start message.
@@ -107,24 +122,22 @@ class WsgiToAsgiInstance:
         }
 
     @sync_to_async
-    def run_wsgi_app(self, message):
+    def run_wsgi_app(self, body):
         """
         Called in a subthread to run the WSGI app. We encapsulate like
         this so that the start_response callable is called in the same thread.
         """
-        # Translate the scope and incoming message into a WSGI environ
-        environ = self.build_environ(self.scope, message)
+        # Translate the scope and incoming request body into a WSGI environ
+        environ = self.build_environ(self.scope, body)
         # Run the WSGI app
         for output in self.wsgi_application(environ, self.start_response):
             # If this is the first response, include the response headers
             if not self.response_started:
                 self.response_started = True
                 self.sync_send(self.response_start)
-            self.sync_send({
-                "type": "http.response.body",
-                "body": output,
-                "more_body": True,
-            })
+            self.sync_send(
+                {"type": "http.response.body", "body": output, "more_body": True}
+            )
         # Close connection
         if not self.response_started:
             self.response_started = True
