@@ -77,6 +77,28 @@ async def test_sync_to_async_method_decorator():
     assert result == 44
 
 
+@pytest.mark.asyncio
+async def test_sync_to_async_method_self_attribute():
+    """
+    Tests sync_to_async on a method copies __self__
+    """
+
+    # Define sync function
+    class TestClass:
+        def test_method(self):
+            time.sleep(0.1)
+            return 45
+
+    # Check it works right
+    instance = TestClass()
+    method = sync_to_async(instance.test_method)
+    result = await method()
+    assert result == 45
+
+    # Check __self__ has been copied
+    assert method.__self__ == instance
+
+
 @pytest.mark.asyncio
 async def test_async_to_sync_to_async():
     """
@@ -337,3 +359,15 @@ class ASGITest(TestCase):
     @async_to_sync
     async def test_wrapped_case_is_collected(self):
         self.assertTrue(True)
+
+
+def test_sync_to_async_detected_as_coroutinefunction():
+    """
+    Tests that SyncToAsync functions are detected as coroutines.
+    """
+
+    def sync_func():
+        return
+
+    assert not asyncio.iscoroutinefunction(sync_to_async)
+    assert asyncio.iscoroutinefunction(sync_to_async(sync_func))
